@@ -3,12 +3,160 @@ import jmespath
 import hashlib
 from faker import Faker
 import time
-import datetime
+from datetime import datetime
 import pymysql
 from loguru import logger
 from functools import wraps
+import os
+import json
+import re
+from xml.dom.minidom import parse
+import xmltodict
+from jsonpath import jsonpath
+import wget
 
-def getFromList(input_list=[],list_index=0)
+
+
+def email_sender(host="smtp.163.com",sender="xxx@163.com",auth_code='xxx',reciever=['xxx','xxx'],content="email content"):
+    import smtplib
+    import email
+
+    # 负责将多个对象集合起来
+    from email.mime.multipart import MIMEMultipart
+    from email.header import Header
+
+    # SMTP服务器,这里使用163邮箱
+    mail_host = host
+    # 发件人邮箱
+    mail_sender = sender
+    # 邮箱授权码,注意这里不是邮箱密码,如何获取邮箱授权码,请看本文最后教程
+    mail_license = auth_code
+    # 收件人邮箱，可以为多个收件人
+    mail_receivers = reciever
+
+    mm = MIMEMultipart('related')
+    # 邮件正文内容
+    body_content = content
+    # 构造文本,参数1：正文内容，参数2：文本格式，参数3：编码方式
+    message_text = MIMEText(body_content,"plain","utf-8")
+    # 向MIMEMultipart对象中添加文本对象
+    mm.attach(message_text)
+
+    # 创建SMTP对象
+    stp = smtplib.SMTP()
+    # 设置发件人邮箱的域名和端口，端口地址为25
+    stp.connect(mail_host, 25)  
+    # set_debuglevel(1)可以打印出和SMTP服务器交互的所有信息
+    stp.set_debuglevel(1)
+    # 登录邮箱，传递参数1：邮箱地址，参数2：邮箱授权码
+    stp.login(mail_sender,mail_license)
+    # 发送邮件，传递参数1：发件人邮箱地址，参数2：收件人邮箱地址，参数3：把邮件内容格式改为str
+    stp.sendmail(mail_sender, mail_receivers, mm.as_string())
+    print("邮件发送成功")
+    # 关闭SMTP对象
+    stp.quit()
+
+
+
+def download2Dest(url,destDir,fileName):
+    try:
+        if not os.path.exists(destDir):
+            os.mkdir(destDir)
+        wget.download(url,out=destDir+os.sep+fileName)
+        return True
+    except Exception as e:
+        logger.error(e)
+        return False
+    return True
+
+
+def findBy(oriStr,flag="",pattern=""):
+    if flag=="json":
+        if isJson(str(oriStr)):
+            try:
+                allStr = json.loads(oriStr)
+                flag=jmespath.search(pattern,allStr)
+                if flag:
+                    return flag
+                else:
+                    return False
+            except Exception as e:
+                logger.error('json 解析失败')
+                return False  
+    elif flag=="regx":
+        try:
+            allStr = oriStr
+            flag=re.compile(pattern)
+            res=flag.findall(allStr)
+            targetStr="".join(res)
+            if targetStr:
+                return targetStr
+            else:
+                return False
+        except Exception as e:
+            logger.error('regx 解析失败')
+            return False
+    else:
+        logger.error(f'无此{flag}解析方式')
+
+
+
+def jsonFilter(oriStr,flag,pattern,key):
+    if isJson(str(oriStr)):
+        try:
+            allStr = json.loads(oriStr)
+            flag=jmespath.search(pattern,allStr)
+            if flag in key:
+                return True, allStr
+        except Exception as e:
+            logger.error('json 解析失败')
+            logger.error(f'{allStr}')
+            return False, allStr
+    else:
+        logger.error(f"{allStr} isn't a string.")
+        return False, allStr
+
+def regxFilter(oriStr,flag,pattern,key):
+    try:
+        allStr = oriStr
+        flag=re.compile(pattern)
+        res=flag.findall(allStr)
+        if key in "".join(res):
+            return True, allStr
+        return False, allStr
+    except Exception as e:
+        logger.error('regx 解析失败')
+        logger.error(f'{allStr}') 
+        return False, allStr
+
+def containFilter(oriStr,key):
+    try:
+        if key in oriStr:
+            return True, oriStr
+        else:
+            return False, oriStr
+    except Exception as e:
+        logger.error('contain 错误')
+        return False, oriStr
+
+def checkStr(oriStr,flag,pattern,key):
+        if flag=="json":
+            if jsonFilter(oriStr, flag, pattern, key)[0]!=False:
+                return True
+            else:
+                return False
+        elif flag=="regx":
+            if regxFilter(oriStr, flag, pattern, key)[0]!=False:
+                return True
+            else:
+                return False
+        elif flag=="contain":
+            if containFilter(oriStr, key)[0]!=False:
+                return True
+            else:
+                return False
+
+def getFromList(input_list=[],list_index=0):
     if len(input_list)==1:
         return input_list[0]
     else:
@@ -38,6 +186,19 @@ def multi2Single(*args):
     return ret
 
 
+#判断是否为json串
+def isJson(oriStr):
+    if isinstance(oriStr,str):
+        try:
+            targetStr=json.loads(oriStr)
+        except ValueError as e:
+            return False
+        return True
+    else:
+        logger.error(f'{oriStr} not a string')
+        return False
+
+
 #获取数组维数
 num = 1
 def getDimons(mylist):
@@ -312,14 +473,55 @@ def getTimeStamp():
     '''
     return round(time.time() * 1000)
 
+def formatStr2Timestamp(timeStr: str, format_style='%Y-%m-%d %H:%M:%S.%f'):
+    """
+    timeStr:"2022-01-01 08:08:08.999"
+    """
+    # time_array = time.strptime(timeStr, format_type)
+    # return int(time.mktime(time_array)) * 1000
 
-def getDateTime(format="%Y-%m-%d %H:%M:%S"):
+    datetime_obj = datetime.strptime(timeStr, format_style)
+    ret_stamp = int(time.mktime(datetime_obj.timetuple()) * 1000.0 + datetime_obj.microsecond / 1000.0)
+    return ret_stamp
+
+
+def timeStampStr2FormatTime(timestamp: int, format_style='%Y-%m-%d %H:%M:%S.%f'):
+    """"
+    timestampStr:1648741273998
+    """
+    # return time.strftime(format_type,time.localtime(timestampStr))
+
+    timestamp = float(timestamp) / 1000
+    ret_datetime = datetime.utcfromtimestamp(timestamp).strftime(format_style)
+    return ret_datetime
+
+
+def timestampsCompare(bigOne: int, smarllOne: int):
+    """
+    bigOne:1648792986000
+    smarllOne:1648792990000
+    """
+    big = datetime.utcfromtimestamp(float(bigOne) / 1000).strftime('%Y-%m-%d %H:%M:%S.%f')
+    bigt = datetime.strptime(big, '%Y-%m-%d %H:%M:%S.%f')
+    # print(bigt)
+    small = datetime.utcfromtimestamp(float(smarllOne) / 1000).strftime('%Y-%m-%d %H:%M:%S.%f')
+    smallt = datetime.strptime(small, '%Y-%m-%d %H:%M:%S.%f')
+    # print(smallt)
+    dalta = bigt - smallt
+    # return dalta.milliseconds
+    return dalta.total_seconds()
+
+
+
+
+
+def getDateTime(format="%Y-%m-%d %H:%M:%S.%f"):
     '''
 
     :param format: time format
     :return: format time
     '''
-    return datetime.datetime.strftime(datetime.datetime.now(), format)
+    return datetime.strftime(datetime.now(), format)
 
 def jsonfile_to_obj(json_file_path):
     # import json
@@ -404,3 +606,74 @@ def makeCase(xmindPath,excelName):
                 ws.cell(row,index+1,value=value)
         row=row+1
     wb.save(excelName)   
+
+
+# def swagger2jmeter(url):
+#     from swaggerjmx.convert import conversion
+#     from swaggerjmx.settings import Settings as ST
+#     import traceback
+#     #  swagger_url
+#     try:
+#         ST.swagger_url = url
+#         #  report_path
+#         ST.report_path = 'jmx'
+#         # 开始转换
+#         conversion()
+#         # print(os.path.abspath(os.path.dirname(__file__))+os.sep+'jmx'+os.sep)
+#         upperDir=os.path.abspath(os.path.dirname(__file__))+os.sep+'jmx'+os.sep
+#         for x,y,z in os.walk(os.path.abspath(os.path.dirname(__file__))+os.sep+'jmx'+os.sep):
+#             return upperDir+"".join(z)
+#     except Exception as e:
+#         print(traceback.format_exc())
+#         return "switch to jmeter script error"
+
+
+def parseJmeterXml(filePath='/Users/oupeng/ssoHttp.jmx'):
+    try:
+        resList=[]
+        with open(filePath,mode='r',encoding='utf8') as xf:
+            doc=xmltodict.parse(xf.read())
+            # print(doc['jmeterTestPlan']['hashTree']['hashTree']['Arguments']['collectionProp']['elementProp']['stringProp'])
+            json_result=json.loads(json.dumps(doc,ensure_ascii=False))
+            # print(json_result)
+            args=jsonpath(json_result, '$..Arguments.collectionProp.elementProp[*].stringProp')
+            # print(args)
+            
+            for arg in args:
+                tempList=[]
+                for one in arg:
+                    if one['@name']=='Argument.name':
+                        tempList.append(one['#text'])
+                    elif one['@name']=='Argument.value':
+                        if "${__P" in one['#text']:
+                            tempList.append(one['#text'].strip('${__P(').strip(')}').split(',')[1])
+                        else:
+                            tempList.append(one['#text'])
+                    elif one['@name']=='Argument.desc':
+                        tempList.append(one['#text'])
+                if len(tempList)>2:
+                    # print(tempList)
+                    if tempList not in resList:
+                        resList.append(tempList)
+            # print(resList)
+        if len(resList)>0:
+            return resList
+        else:
+            return False
+    except Exception as e:
+        print(e)
+        
+
+        
+    # return paramList
+
+
+
+if __name__=='__main__':
+    # print(CFacker().get_it("month"))
+    # print(swagger2jmeter("http://192.168.118.168:12812/space/v2/api-docs"))
+    # print(formatStr2Timestamp("2022-04-01 08:08:08.345"))
+    # print(timeStampStr2FormatTime(1648741273123))
+    # timestampsCompare(1648741273123, 1648741259023)
+    # print(parseJmeterXml())
+    download2Dest("https://th.bing.com/th/id/OIP.hN3O2jHxpAjx7N_J9vmBYAHaBf?w=294&h=70&c=7&r=0&o=5&pid=1.7", os.path.expanduser('~')+os.sep+"abc.png")
\ No newline at end of file
